
;**************************************************************************
;*                             Veza                                       *
;**************************************************************************
;*          4 ziarovky connected to RB0 - RB3                             *
;**************************************************************************
;*          v 1.05 - 17.10.2013                                           *
;**************************************************************************
;*          Hardw.: PIC16F628A used                                       *
;*          OSC.......: Internal osc. used     POWER.....:  18V + 7805    *
;**************************************************************************

W                            EQU     H'0000'
F                            EQU     H'0001'

;----- Register Files------------------------------------------------------

INDF                         EQU     H'0000'
TMR0                         EQU     H'0001'
PCL                          EQU     H'0002'
STATUS                       EQU     H'0003'
FSR                          EQU     H'0004'
PORTA                        EQU     H'0005'
PORTB                        EQU     H'0006'

INTCON                       EQU     H'000B'
OPTION_REG                   EQU     H'0081'
TRISA                        EQU     H'0085'
TRISB                        EQU     H'0086'
CMCON                        EQU     H'001F'

PCON                         EQU     H'008E'
PIR1                         EQU     H'000C'	;bank0
PIE1                         EQU     H'008C'	;bank1
T2CON                        EQU     H'0012'	;bank0

TMR2                         EQU     H'0011'	;bank0
PR2			     EQU     H'0092'	;bank1

;----- STATUS Bits --------------------------------------------------------
IRP                          EQU     H'0007'
RP1                          EQU     H'0006'
RP0                          EQU     H'0005'
NOT_TO                       EQU     H'0004'
NOT_PD                       EQU     H'0003'
Z                            EQU     H'0002'
DC                           EQU     H'0001'
C                            EQU     H'0000'

;==========================================================================
;
;       RAM Definition
;
;==========================================================================

    __MAXRAM H'01FF'
    __BADRAM H'07'-H'09', H'0D', H'13'-H'14', H'1B'-H'1E'
    __BADRAM H'87'-H'89', H'8D', H'8F'-H'91', H'93'-H'97', H'9E'
    __BADRAM H'105', H'107'-H'109', H'10C'-H'11F', H'150'-H'16F'
    __BADRAM H'185', H'187'-H'189', H'18C'-H'1EF'

;==========================================================================
;
;       Configuration Bits
;
;==========================================================================

_BODEN_ON                    EQU     H'3FFF'
_BODEN_OFF                   EQU     H'3FBF'
_CP_ALL                      EQU     H'03FF'
_CP_75                       EQU     H'17FF'
_CP_50                       EQU     H'2BFF'
_CP_OFF                      EQU     H'3FFF'
_DATA_CP_ON                  EQU     H'3EFF'
_DATA_CP_OFF                 EQU     H'3FFF'
_PWRTE_OFF                   EQU     H'3FFF'
_PWRTE_ON                    EQU     H'3FF7'
_WDT_ON                      EQU     H'3FFF'
_WDT_OFF                     EQU     H'3FFB'
_LVP_ON                      EQU     H'3FFF'
_LVP_OFF                     EQU     H'3F7F'
_MCLRE_ON                    EQU     H'3FFF'
_MCLRE_OFF                   EQU     H'3FDF'
_ER_OSC_CLKOUT               EQU     H'3FFF'
_ER_OSC_NOCLKOUT             EQU     H'3FFE'
_INTRC_OSC_CLKOUT            EQU     H'3FFD'
_INTRC_OSC_NOCLKOUT          EQU     H'3FFC'
_EXTCLK_OSC                  EQU     H'3FEF'
_LP_OSC                      EQU     H'3FEC'
_XT_OSC                      EQU     H'3FED'
_HS_OSC                      EQU     H'3FEE'

	__CONFIG        _BODEN_OFF & _CP_OFF & _DATA_CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_OFF & _INTRC_OSC_NOCLKOUT

;==========================================================================
;       Variable Definition
;==========================================================================
;INPUTS
SW1		EQU     H'00'		;SW1 is triggering RA0
SW2		EQU     H'01'		;SW2 is triggering RA1
SW3		EQU     H'02'		;SW3 is triggering RA2
SW4		EQU     H'03'		;SW4 is triggering RA3
TIMER1		EQU	H'20'		;Used in delay routine
TIMER2		EQU	H'21'		; "	"	"	
TIMER3		equ	H'22'
TIMER4		equ	H'23'
k1r		equ	H'24'
k2g		equ	H'25'
TIMER11		equ	H'26'
TIMER12		equ	H'27'
TIMER13		equ	H'28'
sleep_act	equ	H'29'
k1r_save	equ	H'2A'
k2g_save	equ	H'2B'
ftf		equ	H'2C'

RB0_0		EQU	H'2D'
RB0_1		EQU	H'2E'
RB0e		EQU	H'2F'
RBxs		EQU	H'30'

RB1_0		EQU	H'31'
RB1_1		EQU	H'32'
RB1e		EQU	H'33'

RB2_0		EQU	H'34'
RB2_1		EQU	H'35'
RB2e		EQU	H'36'

RB3_0		EQU	H'37'
RB3_1		EQU	H'38'
RB3e		EQU	H'39'
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	RESET		;goto RESET routine when boot.


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

RESET		MOVLW	B'00000111'	;Disable Comparator module's
		MOVWF	CMCON
		;
		BSF	STATUS,RP0	;Switch to register bank 1
					;Disable pull-ups
					;INT on rising edge
					;TMR0 to CLKOUT
					;TMR0 Incr low2high trans.
					;Prescaler assign to Timer0
					;Prescaler rate is 1:256
		MOVLW	B'01010111'	;Set PIC options (See datasheet).

;RBPU bit (bit 7 of option reg); if RBPU eq 0 --> pull-ups are active....

		MOVWF	OPTION_REG	;Write the OPTION register.
		;
		CLRF	INTCON		;Disable interrupts

		MOVLW	B'11110000'	;RB4 - RB7: input
		MOVWF	TRISB
		MOVLW	B'11111111'	;RAx- input
		MOVWF	TRISA

		bsf	PCON,D'003'	;set 4MHz oscilator frequency

		movlw	D'200'		;stop value for TIMER2
		movwf	PR2

		BCF	STATUS,RP0	;Switch Back to reg. Bank 0

;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
main1		clrf	PORTB
		call	dr1

		bsf	PORTB,D'000'
		call	dr1

		bcf	PORTB,D'000'
		bsf	PORTB,D'001'
		call	dr1

		bcf	PORTB,D'001'
		bsf	PORTB,D'002'
		call	dr1

		bcf	PORTB,D'002'
		bsf	PORTB,D'003'
		call	dr1

		goto	main1

;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
main2		clrf	PORTB
		call	dr2

		bsf	PORTB,D'000'
		call	dr2

		bsf	PORTB,D'001'
		call	dr2

		bsf	PORTB,D'002'
		call	dr2

		bsf	PORTB,D'003'
		call	dr2

		goto	main2

;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
main3		clrf	PORTB
		call	dr3

		bsf	PORTB,D'003'
		call	dr3

		goto	main3

;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
main4		bcf	PORTB,D'000'
		bcf	PORTB,D'002'
		bsf	PORTB,D'001'
		bsf	PORTB,D'003'
		call	dr4

		bsf	PORTB,D'000'
		bsf	PORTB,D'002'
		bcf	PORTB,D'001'
		bcf	PORTB,D'003'
		call	dr4

		goto	main4

;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
;PWM effect
main5		nop
;Step 1 --------------------------------------
		movlw	D'128'
		movwf	RB0_0
		movlw	D'005'
		movwf	RB0_1

		movlw	D'128'
		movwf	RB1_0
		movlw	D'005'
		movwf	RB1_1

		movlw	D'128'
		movwf	RB2_0
		movlw	D'005'
		movwf	RB2_1

		movlw	D'128'
		movwf	RB3_0
		movlw	D'005'
		movwf	RB3_1
		call	PWM_exec5
;Step 2 --------------------------------------
		movlw	D'128'
		movwf	RB0_0
		movlw	D'050'
		movwf	RB0_1

		movlw	D'128'
		movwf	RB1_0
		movlw	D'050'
		movwf	RB1_1

		movlw	D'128'
		movwf	RB2_0
		movlw	D'050'
		movwf	RB2_1

		movlw	D'128'
		movwf	RB3_0
		movlw	D'050'
		movwf	RB3_1

		call	PWM_exec5
;Step 3 --------------------------------------
		movlw	D'128'
		movwf	RB0_0
		movlw	D'128'
		movwf	RB0_1

		movlw	D'128'
		movwf	RB1_0
		movlw	D'128'
		movwf	RB1_1

		movlw	D'128'
		movwf	RB2_0
		movlw	D'128'
		movwf	RB2_1


		movlw	D'128'
		movwf	RB3_0
		movlw	D'128'
		movwf	RB3_1

		call	PWM_exec5
;Step 4 --------------------------------------
		movlw	D'128'
		movwf	RB0_0
		movlw	D'255'
		movwf	RB0_1

		movlw	D'128'
		movwf	RB1_0
		movlw	D'255'
		movwf	RB1_1

		movlw	D'128'
		movwf	RB2_0
		movlw	D'255'
		movwf	RB2_1

		movlw	D'128'
		movwf	RB3_0
		movlw	D'255'
		movwf	RB3_1

		call	PWM_exec5
;Step 5 --------------------------------------
		movlw	D'001'
		movwf	RB0_0
		movlw	D'255'
		movwf	RB0_1

		movlw	D'001'
		movwf	RB1_0
		movlw	D'255'
		movwf	RB1_1

		movlw	D'001'
		movwf	RB2_0
		movlw	D'255'
		movwf	RB2_1

		movlw	D'001'
		movwf	RB3_0
		movlw	D'255'
		movwf	RB3_1

		call	PWM_exec5

		goto	main5

;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
main6		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		call	dr6

		bcf	PORTB,D'000'
		bcf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		call	dr6

		goto	main6
;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
;PWM effect
main7		movlw	D'128'
		movwf	RB0_0
		movwf	RB1_0
		movwf	RB2_0
		movwf	RB3_0

		movlw	D'000'
		movwf	RB0_1
		movlw	D'255'
		movwf	RB1_1
		movlw	D'000'
		movwf	RB2_1
		movlw	D'255'
		movwf	RB3_1

main72		call	dr7
		movlw	D'010'
		addwf	RB0_1,1
		decf	RB1_1,1
		decf	RB1_1,1
		decf	RB1_1,1
		decf	RB1_1,1
		decf	RB1_1,1
		addwf	RB2_1,1
		decf	RB3_1,1
		decf	RB3_1,1
		decf	RB3_1,1
		decf	RB3_1,1
		decf	RB3_1,1

		goto	main72

;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
main8		bsf	PORTB,D'000'
		bcf	PORTB,D'001'
		bcf	PORTB,D'002'
		bsf	PORTB,D'003'
		call	dr8

		bcf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bcf	PORTB,D'003'
		call	dr8

		goto	main8

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;		*********************************************
;		*               delay routine               *
;		*********************************************

;-------------------------------------------------------------------
dr1		movlw	D'002'
		movwf	TIMER3
del_loop10	MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP11	MOVLW   D'255'
		MOVWF   TIMER1

;sw test
		btfss	PORTA,D'000'
		goto	main2


DEL_LOOP12	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP12
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP11
		decfsz	TIMER3,F
		goto	del_loop10
		return
;-------------------------------------------------------------------
dr2		movlw	D'002'
		movwf	TIMER3
del_loop20	MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP21	MOVLW   D'255'
		MOVWF   TIMER1

;sw test
		btfsc	PORTA,D'000'
		goto	main3


DEL_LOOP22	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP22
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP21
		decfsz	TIMER3,F
		goto	del_loop20
		return

;-------------------------------------------------------------------
dr3		movlw	D'002'
		movwf	TIMER3
del_loop30	MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP31	MOVLW   D'255'
		MOVWF   TIMER1
;sw test
		btfss	PORTA,D'000'
		goto	main4

DEL_LOOP32	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP32
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP31
		decfsz	TIMER3,F
		goto	del_loop30
		return

;-------------------------------------------------------------------
dr4		movlw	D'002'
		movwf	TIMER3
del_loop40	MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP41	MOVLW   D'255'
		MOVWF   TIMER1
;sw test
		btfsc	PORTA,D'000'
		goto	main5

DEL_LOOP42	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP42
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP41
		decfsz	TIMER3,F
		goto	del_loop40
		return

;-------------------------------------------------------------------
dr5		goto	PWM_exec5
;-------------------------------------------------------------------
;-------------------------------------------------------------------
;-------------------------------------------------------------------
dr6		movlw	D'002'
		movwf	TIMER3
del_loop60	MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP61	MOVLW   D'255'
		MOVWF   TIMER1
;sw test
		btfsc	PORTA,D'000'
		goto	main7

DEL_LOOP62	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP62
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP61
		decfsz	TIMER3,F
		goto	del_loop60
		return
;-------------------------------------------------------------------
dr7		goto	PWM_exec7
;-------------------------------------------------------------------

;-------------------------------------------------------------------
dr8		movlw	D'002'
		movwf	TIMER3
del_loop80	MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP81	MOVLW   D'255'
		MOVWF   TIMER1
;sw test
		btfsc	PORTA,D'000'
		goto	main1

DEL_LOOP82	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP82
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP81
		decfsz	TIMER3,F
		goto	del_loop80
		return

;-------------------------------------------------------------------
;-------------------------------------------------------------------
;-------------------------------------------------------------------
PWM_exec5	nop

;--------------------------------------------------
		movlw	D'002'
		movwf	TIMER3
del_loop50	MOVLW   D'025'
		MOVWF   TIMER2
DEL_LOOP51	MOVLW   D'255'
		MOVWF   TIMER1
;sw test
		btfss	PORTA,D'000'
		goto	main6

DEL_LOOP52	DECFSZ  TIMER1,F
                GOTO    pwmi5
		DECFSZ  TIMER2,F
                GOTO    DEL_LOOP51
		decfsz	TIMER3,F
		goto	del_loop50
		return

pwmi5		call	pwmi
		goto	DEL_LOOP52
;-------------------------------------------------------------------
PWM_exec7	nop

;--------------------------------------------------
		movlw	D'002'
		movwf	TIMER3
del_loop70	MOVLW   D'025'
		MOVWF   TIMER2
DEL_LOOP71	MOVLW   D'255'
		MOVWF   TIMER1
;sw test
		btfss	PORTA,D'000'
		goto	main8

DEL_LOOP72	DECFSZ  TIMER1,F
                GOTO    pwmi7
		DECFSZ  TIMER2,F
                GOTO    DEL_LOOP71
		decfsz	TIMER3,F
		goto	del_loop70
		return

pwmi7		call	pwmi
		goto	DEL_LOOP72
;-----------------------------------------------
;-----------------------------------------------
;PWM logic
pwmi		nop
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;RB0 -------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
		decf	RB0e,1
		btfss	STATUS,Z
		goto	pwmi_rb0f
		btfsc	RBxs,D'000'
		goto	pwm_rb0_0
		goto	pwm_rb0_1
;-----------------------------------------------
;RB0 was on 1; now we have to switch off
pwm_rb0_0	bcf	RBxs,D'000'
		bcf	PORTB,D'000'
		movf	RB0_0,0
		movwf	RB0e
		goto	pwmi_rb0f
;-----------------------------------------------
;RB0 was on 0; now we have to switch on
pwm_rb0_1	bsf	RBxs,D'000'
		bsf	PORTB,D'000'
		movf	RB0_1,0
		movwf	RB0e
		goto	pwmi_rb0f
;-----------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;RB1 -------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
pwmi_rb0f	nop

		decf	RB1e,1
		btfss	STATUS,Z
		goto	pwmi_rb1f
		btfsc	RBxs,D'001'
		goto	pwm_rb1_0
		goto	pwm_rb1_1
;-----------------------------------------------
;RB1 was on 1; now we have to switch off
pwm_rb1_0	bcf	RBxs,D'001'
		bcf	PORTB,D'001'
		movf	RB1_0,0
		movwf	RB1e
		goto	pwmi_rb1f
;-----------------------------------------------
;RB1 was on 0; now we have to switch on
pwm_rb1_1	bsf	RBxs,D'001'
		bsf	PORTB,D'001'
		movf	RB1_1,0
		movwf	RB1e
		goto	pwmi_rb1f

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;RB2 -------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
pwmi_rb1f	nop

		decf	RB2e,1
		btfss	STATUS,Z
		goto	pwmi_rb2f
		btfsc	RBxs,D'002'
		goto	pwm_rb2_0
		goto	pwm_rb2_1
;-----------------------------------------------
;RB2 was on 1; now we have to switch off
pwm_rb2_0	bcf	RBxs,D'002'
		bcf	PORTB,D'002'
		movf	RB2_0,0
		movwf	RB2e
		goto	pwmi_rb2f
;-----------------------------------------------
;RB2 was on 0; now we have to switch on
pwm_rb2_1	bsf	RBxs,D'002'
		bsf	PORTB,D'002'
		movf	RB2_1,0
		movwf	RB2e
		goto	pwmi_rb2f

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;RB3 -------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
pwmi_rb2f	nop

		decf	RB3e,1
		btfss	STATUS,Z
		goto	pwmi_rb3f
		btfsc	RBxs,D'003'
		goto	pwm_rb3_0
		goto	pwm_rb3_1
;-----------------------------------------------
;RB3 was on 1; now we have to switch off
pwm_rb3_0	bcf	RBxs,D'003'
		bcf	PORTB,D'003'
		movf	RB3_0,0
		movwf	RB3e
		goto	pwmi_rb3f
;-----------------------------------------------
;RB3 was on 0; now we have to switch on
pwm_rb3_1	bsf	RBxs,D'003'
		bsf	PORTB,D'003'
		movf	RB3_1,0
		movwf	RB3e
		goto	pwmi_rb3f
;-----------------------------------------------
;-----------------------------------------------
pwmi_rb3f	nop

		return

;-------------------------------------------------------------------
;-------------------------------------------------------------------
;-------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

		END